"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.jupClient = void 0;
const web3_js_1 = require("@solana/web3.js");
const logger_1 = require("./logger");
const jupClient = (connection, funder, log = logger_1.consoleLogger) => {
    return {
        createTradeInstructions(f) {
            var _a;
            return __awaiter(this, void 0, void 0, function* () {
                const quoteResponse = yield (yield fetch(`https://quote-api.jup.ag/v6/quote?inputMint=EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v
&outputMint=rndrizKT3MK1iimdxRdWabcF7Zg7AR5T4nud4EkHBof
&amount=${f.renderAmount}
&slippageBps=${(_a = f.slippage) !== null && _a !== void 0 ? _a : '50'}${f.maxAccounts ? `&maxAccounts=${f.maxAccounts}` : ''}`)).json();
                log.debug(`renderAmt ${f.renderAmount} quoteResponse ${JSON.stringify(quoteResponse)}`);
                const instructions = yield (yield fetch('https://quote-api.jup.ag/v6/swap-instructions', {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify({
                        // quoteResponse from /quote api
                        quoteResponse,
                        userPublicKey: funder.publicKey.toBase58(),
                        dynamicComputeUnitLimit: true, // allow dynamic compute limit instead of max 1,400,000
                        // custom priority fee - Jup will calculate
                        prioritizationFeeLamports: 'auto' // or custom lamports: 1000
                        // feeAccount is optional. Use if you want to charge a fee.  feeBps must have been passed in /quote API.
                        // feeAccount: "fee_account_public_key"
                    })
                })).json();
                if (instructions.error) {
                    throw new Error("Failed to get swap instructions: " + instructions.error);
                }
                log.info(`instructions payload ${JSON.stringify(instructions)}`);
                const { 
                //tokenLedgerInstruction, // If you are using `useTokenLedger = true`.
                computeBudgetInstructions, // The necessary instructions to setup the compute budget.
                setupInstructions, // Setup missing ATA for the users.
                swapInstruction: swapInstructionPayload, // The actual swap instruction.
                //cleanupInstruction, // Unwrap the SOL if `wrapAndUnwrapSol = true`.
                addressLookupTableAddresses, // The lookup table addresses that you can use if you are using versioned transaction.
                prioritizationFeeLamports, // jup calced prio fee
                 } = instructions;
                const deserializeInstruction = (instruction) => {
                    return new web3_js_1.TransactionInstruction({
                        programId: new web3_js_1.PublicKey(instruction.programId),
                        keys: instruction.accounts.map((key) => ({
                            pubkey: new web3_js_1.PublicKey(key.pubkey),
                            isSigner: key.isSigner,
                            isWritable: key.isWritable,
                        })),
                        data: Buffer.from(instruction.data, "base64"),
                    });
                };
                const getAddressLookupTableAccounts = (keys) => __awaiter(this, void 0, void 0, function* () {
                    const addressLookupTableAccountInfos = yield connection.getMultipleAccountsInfo(keys.map((key) => new web3_js_1.PublicKey(key)));
                    return addressLookupTableAccountInfos.reduce((acc, accountInfo, index) => {
                        const addressLookupTableAddress = keys[index];
                        if (accountInfo) {
                            const addressLookupTableAccount = new web3_js_1.AddressLookupTableAccount({
                                key: new web3_js_1.PublicKey(addressLookupTableAddress),
                                state: web3_js_1.AddressLookupTableAccount.deserialize(accountInfo.data),
                            });
                            acc.push(addressLookupTableAccount);
                        }
                        return acc;
                    }, new Array());
                });
                const addressLookupTableAccounts = [];
                addressLookupTableAccounts.push(...(yield getAddressLookupTableAccounts(addressLookupTableAddresses)));
                return {
                    instructions: [
                        // uncomment if needed:
                        ...setupInstructions.map(deserializeInstruction),
                        ...computeBudgetInstructions.map(deserializeInstruction),
                        deserializeInstruction(swapInstructionPayload),
                        // uncomment if needed: deserializeInstruction(cleanupInstruction),
                    ],
                    addressLookupTableAccounts,
                };
            });
        }
    };
};
exports.jupClient = jupClient;
//# sourceMappingURL=tradeClient.js.map